==> fess-fess_tags_fess-6.0.0-r1348.tar.gz: fess-fess_tags_fess-6.0.0-r1348/fess-6.0.0/src/main/java/jp/sf/fess/solr/FessHttpSolrServer.java <==
/*
 * Copyright 2009-2012 the Fess Project and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package jp.sf.fess.solr;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import jp.sf.fess.FessSystemException;
import jp.sf.fess.helper.SystemHelper;
import jp.sf.fess.solr.response.ReplicationResponse;

import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.io.FileUtils;
import org.apache.solr.client.solrj.ResponseParser;
import org.apache.solr.client.solrj.impl.BinaryResponseParser;
import org.apache.solr.client.solrj.impl.CommonsHttpSolrServer;
import org.seasar.framework.container.SingletonS2Container;
import org.seasar.framework.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FessHttpSolrServer extends CommonsHttpSolrServer implements
        FessSolrServer {

    private static final long serialVersionUID = 1L;

    private static final Logger logger = LoggerFactory
            .getLogger(FessHttpSolrServer.class);

    protected String dataDir;

    protected String solrDataDirName = "fess.solr.data.dir";

    public FessHttpSolrServer(final String solrServerUrl)
            throws MalformedURLException {
        super(new URL(solrServerUrl));
    }

    public FessHttpSolrServer(final String solrServerUrl,
            final HttpClient httpClient) throws MalformedURLException {
        super(new URL(solrServerUrl), httpClient, new BinaryResponseParser(),
                false);
    }

    public FessHttpSolrServer(final String solrServerUrl,
            final HttpClient httpClient, final boolean useMultiPartPost)
            throws MalformedURLException {
        super(new URL(solrServerUrl), httpClient, new BinaryResponseParser(),
                useMultiPartPost);
    }

    public FessHttpSolrServer(final String solrServerUrl,
            final HttpClient httpClient, final ResponseParser parser)
            throws MalformedURLException {
        super(new URL(solrServerUrl), httpClient, parser, false);
    }

    public FessHttpSolrServer(final URL baseURL) {
        super(baseURL, null, new BinaryResponseParser(), false);
    }

    public FessHttpSolrServer(final URL baseURL, final HttpClient client) {
        super(baseURL, client, new BinaryResponseParser(), false);
    }

    public FessHttpSolrServer(final URL baseURL, final HttpClient client,
            final boolean useMultiPartPost) {
        super(baseURL, client, new BinaryResponseParser(), useMultiPartPost);
    }

    public FessHttpSolrServer(final URL baseURL, final HttpClient client,
            final ResponseParser parser, final boolean useMultiPartPost) {
        super(baseURL, client, parser, useMultiPartPost);
    }

    public void setCredentials(final AuthScope authscope,
            final Credentials credentials) {
        getHttpClient().getState().setCredentials(authscope, credentials);
    }

    public String getDataDir() {
        return dataDir;
    }

    public void setDataDir(String dataDir) {
        if (StringUtil.isNotBlank(dataDir)) {
            final String solrDataDir = System.getProperty(solrDataDirName);
            if (solrDataDir != null) {
                dataDir = dataDir.replaceAll("\\$\\{fess.solr.data.dir\\}",
                        solrDataDir);
            }
            if (dataDir.endsWith("/")) {
                this.dataDir = dataDir.replaceAll("/+$", "");
            } else {
                this.dataDir = dataDir;
            }
        } else {
            this.dataDir = dataDir;
        }
    }

    /* (non-Javadoc)
     * @see jp.sf.fess.solr.FessSolrServer#replicate(java.io.File)
     */
    @Override
    public ReplicationResponse replicate(final File snapshotDir) {
        if (snapshotDir == null) {
            throw new FessSystemException("snapshotDir is null.");
        }

        final ReplicationResponse response = new ReplicationResponse();
        response.setRequestUrl(snapshotDir.getAbsolutePath());
        if (StringUtil.isBlank(getDataDir())) {
            throw new FessSolrReplicationException("Skipped a replication. "
                    + "DataDir is a blank. BaseURL is " + getBaseURL());
        }

        if (!snapshotDir.exists()) {
            throw new FessSolrReplicationException(
                    "Invalid snapshot directory: "
                            + snapshotDir.getAbsolutePath());
        }

        final long startTime = System.currentTimeMillis();

        final SystemHelper systemHelper = SingletonS2Container
                .getComponent("systemHelper");
        final String sessionId = systemHelper.getSessionId();

        final File indexDir = new File(getDataDir(), "index");
        final File newIndexDir = new File(getDataDir(), "index");
        final File newSnapshotDir = new File(getDataDir(), "index.new."
                + sessionId);
        final File oldSnapshotDir = new File(getDataDir(), "index.old."
                + sessionId);

        if (logger.isInfoEnabled()) {
            logger.info("snapshotDir: " + snapshotDir.getAbsolutePath());
            logger.info("indexDir: " + indexDir.getAbsolutePath());
        }

        if (logger.isDebugEnabled()) {
            logger.debug("newIndexDir: " + newIndexDir.getAbsolutePath());
            logger.debug("newSnapshotDir: " + newSnapshotDir.getAbsolutePath());
            logger.debug("oldSnapshotDir: " + oldSnapshotDir.getAbsolutePath());
        }

        try {
            try {
                FileUtils.copyDirectory(snapshotDir, newSnapshotDir);
            } catch (final IOException e) {
                throw new FessSolrReplicationException("Could not copy "
                        + snapshotDir.getAbsolutePath() + " to"
                        + newSnapshotDir.getAbsolutePath());
            }
            if (!indexDir.renameTo(oldSnapshotDir)) {
                throw new FessSolrException("Could not rename "
                        + indexDir.getAbsolutePath() + " to "
                        + oldSnapshotDir.getAbsolutePath());
            }
            if (!newSnapshotDir.renameTo(newIndexDir)) {
                // TODO should rollback above?
                throw new FessSolrException("Could not rename "
                        + newSnapshotDir.getAbsolutePath() + " to "
                        + newIndexDir.getAbsolutePath());
            }
        } catch (final FessSolrException e) {
            throw e;
        } catch (final Exception e) {
            throw new FessSolrException("Failed to replicate index data.", e);
        }
        try {
            FileUtils.deleteDirectory(oldSnapshotDir);
        } catch (final Exception e) {
            oldSnapshotDir.deleteOnExit();
            logger.warn("Could not delete old index directory: "
                    + oldSnapshotDir.getAbsolutePath(), e);
        }
        response.setElapsedTime(System.currentTimeMillis() - startTime);
        return response;
    }

    public void setAuthenticationPreemptive(final boolean value) {
        getHttpClient().getParams().setAuthenticationPreemptive(value);
    }

    public String getSolrDataDirName() {
        return solrDataDirName;
    }

    public void setSolrDataDirName(final String solrDataDirName) {
        this.solrDataDirName = solrDataDirName;
    }
}

==> fess-fess_tags_fess-7.0.1-r1373.tar.gz: fess-fess_tags_fess-7.0.1-r1373/fess-7.0.1/src/main/java/jp/sf/fess/solr/FessHttpSolrServer.java <==
/*
 * Copyright 2009-2012 the Fess Project and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package jp.sf.fess.solr;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;

import jp.sf.fess.FessSystemException;
import jp.sf.fess.helper.SystemHelper;
import jp.sf.fess.solr.response.ReplicationResponse;

import org.apache.commons.io.FileUtils;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.client.HttpClient;
import org.apache.http.impl.client.AbstractHttpClient;
import org.apache.solr.client.solrj.ResponseParser;
import org.apache.solr.client.solrj.impl.BinaryResponseParser;
import org.apache.solr.client.solrj.impl.HttpSolrServer;
import org.seasar.framework.container.SingletonS2Container;
import org.seasar.framework.container.annotation.tiger.DestroyMethod;
import org.seasar.framework.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Experimental!<br>
 * HttpSolrServer implementation for Fess.
 * 
 * @author shinsuke
 * 
 */
public class FessHttpSolrServer extends HttpSolrServer implements
        FessSolrServer {

    private static final long serialVersionUID = 1L;

    private static final Logger logger = LoggerFactory
            .getLogger(FessHttpSolrServer.class);

    protected String dataDir;

    protected String solrDataDirName = "fess.solr.data.dir";

    public FessHttpSolrServer(final String solrServerUrl)
            throws MalformedURLException {
        super(solrServerUrl);
    }

    public FessHttpSolrServer(final String solrServerUrl,
            final HttpClient httpClient) throws MalformedURLException {
        super(solrServerUrl, httpClient, new BinaryResponseParser());
    }

    public FessHttpSolrServer(final String solrServerUrl,
            final HttpClient httpClient, final ResponseParser parser)
            throws MalformedURLException {
        super(solrServerUrl, httpClient, parser);
    }

    public void setCredentials(final AuthScope authscope,
            final Credentials credentials) {
        final HttpClient httpClient = getHttpClient();
        if (httpClient instanceof AbstractHttpClient) {
            ((AbstractHttpClient) httpClient).getCredentialsProvider()
                    .setCredentials(authscope, credentials);
        }
    }

    @DestroyMethod
    public void destroy() {
        shutdown();
    }

    public String getDataDir() {
        return dataDir;
    }

    public void setDataDir(String dataDir) {
        if (StringUtil.isNotBlank(dataDir)) {
            final String solrDataDir = System.getProperty(solrDataDirName);
            if (solrDataDir != null) {
                dataDir = dataDir.replaceAll("\\$\\{fess.solr.data.dir\\}",
                        solrDataDir);
            }
            if (dataDir.endsWith("/")) {
                this.dataDir = dataDir.replaceAll("/+$", "");
            } else {
                this.dataDir = dataDir;
            }
        } else {
            this.dataDir = dataDir;
        }
    }

    /* (non-Javadoc)
     * @see jp.sf.fess.solr.FessSolrServer#replicate(java.io.File)
     */
    @Override
    public ReplicationResponse replicate(final File snapshotDir) {
        if (snapshotDir == null) {
            throw new FessSystemException("snapshotDir is null.");
        }

        final ReplicationResponse response = new ReplicationResponse();
        response.setRequestUrl(snapshotDir.getAbsolutePath());
        if (StringUtil.isBlank(getDataDir())) {
            throw new FessSolrReplicationException("Skipped a replication. "
                    + "DataDir is a blank. BaseURL is " + getBaseURL());
        }

        if (!snapshotDir.exists()) {
            throw new FessSolrReplicationException(
                    "Invalid snapshot directory: "
                            + snapshotDir.getAbsolutePath());
        }

        final long startTime = System.currentTimeMillis();

        final SystemHelper systemHelper = SingletonS2Container
                .getComponent("systemHelper");
        final String sessionId = systemHelper.getSessionId();

        final File indexDir = new File(getDataDir(), "index");
        final File newIndexDir = new File(getDataDir(), "index");
        final File newSnapshotDir = new File(getDataDir(), "index.new."
                + sessionId);
        final File oldSnapshotDir = new File(getDataDir(), "index.old."
                + sessionId);

        if (logger.isInfoEnabled()) {
            logger.info("snapshotDir: " + snapshotDir.getAbsolutePath());
            logger.info("indexDir: " + indexDir.getAbsolutePath());
        }

        if (logger.isDebugEnabled()) {
            logger.debug("newIndexDir: " + newIndexDir.getAbsolutePath());
            logger.debug("newSnapshotDir: " + newSnapshotDir.getAbsolutePath());
            logger.debug("oldSnapshotDir: " + oldSnapshotDir.getAbsolutePath());
        }

        try {
            try {
                FileUtils.copyDirectory(snapshotDir, newSnapshotDir);
            } catch (final IOException e) {
                throw new FessSolrReplicationException("Could not copy "
                        + snapshotDir.getAbsolutePath() + " to "
                        + newSnapshotDir.getAbsolutePath(), e);
            }
            if (!indexDir.renameTo(oldSnapshotDir)) {
                throw new FessSolrException("Could not rename "
                        + indexDir.getAbsolutePath() + " to "
                        + oldSnapshotDir.getAbsolutePath());
            }
            if (!newSnapshotDir.renameTo(newIndexDir)) {
                // TODO should rollback above?
                throw new FessSolrException("Could not rename "
                        + newSnapshotDir.getAbsolutePath() + " to "
                        + newIndexDir.getAbsolutePath());
            }
        } catch (final FessSolrException e) {
            throw e;
        } catch (final Exception e) {
            throw new FessSolrException("Failed to replicate index data.", e);
        }
        try {
            FileUtils.deleteDirectory(oldSnapshotDir);
        } catch (final Exception e) {
            oldSnapshotDir.deleteOnExit();
            logger.warn("Could not delete old index directory: "
                    + oldSnapshotDir.getAbsolutePath(), e);
        }
        response.setElapsedTime(System.currentTimeMillis() - startTime);
        return response;
    }

    public String getSolrDataDirName() {
        return solrDataDirName;
    }

    public void setSolrDataDirName(final String solrDataDirName) {
        this.solrDataDirName = solrDataDirName;
    }
}
